@@ -24,11 +24,14 @@
  *
  */
 
+#include <config.h>
 #include <popt.h>
 #include <stdio.h>
 #include <glib.h>
 #include <string.h>
 #include <signal.h>
+#include <time.h>
+#include <unistd.h>
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomeprint/gnome-print-job.h>
 #include <libgnomeprint/gnome-font.h>
@@ -40,6 +43,7 @@
 gboolean options_dump = FALSE;
 gboolean options_dump_full = FALSE;
 gint     options_generate = -1;
+gint     options_max = -1;
 gboolean options_catalog = FALSE;
 gboolean options_number = FALSE;
 gchar*   options_output = NULL;
@@ -60,6 +64,8 @@ static struct poptOption options[] = {
 	  "Return the number of fonts known to gnome-print",  NULL},
 	{ "catalog",   'c', POPT_ARG_NONE,   &options_catalog,   0,
 	  "Generate a font catalog",  NULL},
+	{ "max",       'm', POPT_ARG_INT,    &options_max,     0,
+	  "Max number of fonts to embed",    NULL},
 	{ "output",    'o', POPT_ARG_STRING, &options_output,   0,
 	  "Specify the output file",  NULL},
 	POPT_AUTOHELP
@@ -67,6 +73,43 @@ static struct poptOption options[] = {
 };
 
 
+#define CONFIG_FILE "catalog.GnomePrintConfig"
+
+static GnomePrintConfig *
+my_config_load_from_file (void)
+{
+	GnomePrintConfig *config;
+	FILE *file;
+	gchar *str;
+	gint read, allocated;
+
+	file = fopen (CONFIG_FILE, "r");
+	if (!file) {
+		g_print ("Config not found [%s]\n", CONFIG_FILE);
+		return gnome_print_config_default ();
+	}
+
+#define BLOCK_SIZE 10
+	read = 0;
+	allocated = BLOCK_SIZE;
+	str = g_malloc (allocated);
+	while (TRUE) {
+		gint chars;
+		chars = fread (str + read, sizeof (gchar), allocated - read, file);
+		read += chars;
+		if (chars < 1)
+			break;
+		if ((read + BLOCK_SIZE + 1) > allocated) {
+			allocated += BLOCK_SIZE;
+			str = g_realloc (str, allocated);
+		}
+	}
+	str[read]=0;
+	config = gnome_print_config_from_string (str, 0);
+
+	return config;
+}
+
 static void
 print_font_info (GPFontEntry *entry, GnomePrintContext *gpc)
 {
@@ -193,16 +236,17 @@ generate_font_info (gint num)
 	gpc = gnome_print_job_get_context (job);
 	config = gnome_print_job_get_config (job);
 
-	out_file = options_output ? g_strdup (options_output) : g_strdup_printf ("o%03d.%s", num, options_pdf ? "pdf" : "ps");
-	gnome_print_config_set (config, "Printer", "GENERIC");
-	gnome_print_job_print_to_file (job, out_file);
-	g_free (out_file);
-
 	if (options_pdf) {
 		if (!gnome_print_config_set (config, "Printer", "PDF"))
 			g_print ("Could not set the printer to PDF\n");
+	} else {
+		gnome_print_config_set (config, "Printer", "GENERIC");
 	}
 
+	out_file = options_output ? g_strdup (options_output) : g_strdup_printf ("o%03d.%s", num, options_pdf ? "pdf" : "ps");
+	gnome_print_job_print_to_file (job, out_file);
+	g_free (out_file);
+
 	if (!gnome_print_config_set (config, GNOME_PRINT_KEY_PAPER_SIZE, "USLetter"))
 		g_print ("Could not set the Paper Size\n");
 	
@@ -231,17 +275,123 @@ catalog_endpage (GnomePrintContext *pc)
 	gnome_print_showpage (pc);
 }
 
-#define TOP_MARGIN 20.0
-#define BOTTOM_MARGIN 50.0
+#define TOP_MARGIN    72.0
+#define BOTTOM_MARGIN 72.0
 
 /* The space between the font name and the font info */
-#define DELTA_INFO 5.0
+#define DELTA_INFO 7.0
 #define DELTA_SAMPLE_TEXT 25.0
 
 #define SAMPLE_TEXT_INDENTATION 15.0
 
+#define FONT_BASE "Sans "
+#define _FONT_BASE "Albany AMT "
 #define FONT_NAME_SIZE 8.0
-#define FONT_INFO_SIZE 4.0
+#define FONT_INFO_SIZE 6.0
+#define FONT_NAME FONT_BASE "Regular"
+#define FONT_INFO FONT_BASE "Regular"
+
+#define HEADER_FONT_TITLE_SIZE  14
+#define HEADER_FONT_KEYS_SIZE   8
+#define HEADER_FONT_VALUES_SIZE 8
+#define HEADER_FONT_TITLE  FONT_BASE "Bold"
+#define HEADER_FONT_KEYS   FONT_BASE "Regular"
+#define HEADER_FONT_VALUES FONT_BASE "Italic"
+
+static gchar *
+get_date (void)
+{
+	time_t clock;
+	gchar *s;
+
+	clock = time (NULL);
+	s = ctime (&clock);
+
+	return g_strndup (s, strlen (s) - 1);
+}
+
+static double
+generate_header (GnomePrintConfig *config, GnomePrintContext *gpc, double x, double y)
+{
+	GnomeFont *font1;
+	GnomeFont *font2;
+	GnomeFont *font3;
+	gdouble save, row_size;
+	guchar *s;
+
+	font1 = gnome_font_find_closest (HEADER_FONT_TITLE,  HEADER_FONT_TITLE_SIZE);
+	font2 = gnome_font_find_closest (HEADER_FONT_KEYS,   HEADER_FONT_KEYS_SIZE);
+	font3 = gnome_font_find_closest (HEADER_FONT_VALUES, HEADER_FONT_VALUES_SIZE);
+
+	row_size = 10;
+	
+	gnome_print_setfont (gpc, font1);
+	gnome_print_moveto (gpc, x, y);
+	gnome_print_show (gpc, "GNOME PRINT FONT CATALOG");
+	y -= row_size * 2;
+
+	save = y;
+	gnome_print_setfont (gpc, font2);
+
+	gnome_print_moveto (gpc, x, y);
+	gnome_print_show (gpc, "Libgnomeprint Ver.");
+	y -= row_size;
+	gnome_print_moveto (gpc, x, y);
+	gnome_print_show (gpc, "Hostname");
+	y -= row_size;
+	gnome_print_moveto (gpc, x, y);
+	gnome_print_show (gpc, "Date");
+	y -= row_size;
+	gnome_print_moveto (gpc, x, y);
+	gnome_print_show (gpc, "No. of fonts");
+	y -= row_size;
+	gnome_print_moveto (gpc, x, y);
+	gnome_print_show (gpc, "Media Size");
+	y -= row_size;
+	gnome_print_moveto (gpc, x, y);
+	gnome_print_show (gpc, "Format");
+	y -= row_size;
+
+	gnome_print_setfont (gpc, font3);
+	y = save;
+	x += 72;
+
+	gnome_print_moveto (gpc, x, y);
+	gnome_print_show (gpc, VERSION);
+	y -= row_size;
+	gnome_print_moveto (gpc, x, y);
+	s = g_malloc (256);
+	gethostname (s, 256);
+	gnome_print_show (gpc, s);
+	g_free (s);
+	y -= row_size;
+	gnome_print_moveto (gpc, x, y);
+	s = get_date ();
+	gnome_print_show (gpc, s);
+	g_free (s);
+	y -= row_size;
+	gnome_print_moveto (gpc, x, y);
+	if (options_max == -1)
+		s = g_strdup_printf ("%d", number_of_fonts ());
+	else	
+		s = g_strdup_printf ("%d (max %d)", number_of_fonts (), options_max);
+	gnome_print_show (gpc, s);
+	g_free (s);
+	y -= row_size;
+	gnome_print_moveto (gpc, x, y);
+	s = gnome_print_config_get (config, GNOME_PRINT_KEY_PAPER_SIZE);	
+	gnome_print_show (gpc, s);
+	g_free (s);
+	y -= row_size;
+	gnome_print_moveto (gpc, x, y);
+	gnome_print_show (gpc, options_pdf ? "PDF" : "Postscript");
+ 	
+	g_object_unref (G_OBJECT (font1));
+	g_object_unref (G_OBJECT (font2));
+	g_object_unref (G_OBJECT (font3));
+
+	return y - 40;
+}
 
 static void
 generate_catalog (void)
@@ -256,47 +406,48 @@ generate_catalog (void)
 	gdouble width, height;
 	gint i = 0;
 
-	job = gnome_print_job_new (NULL);
+	config = my_config_load_from_file ();
+	job = gnome_print_job_new (config);
 	gpc = gnome_print_job_get_context (job);
-	config = gnome_print_job_get_config (job);
 
 	gnome_print_config_set (config, "Printer", "GENERIC");
-	gnome_print_job_print_to_file (job, options_output ? options_output :
-				       (options_pdf ? "catlog.pdf" : "catlog.ps"));
 	if (options_pdf) {
 		if (!gnome_print_config_set (config, "Printer", "PDF"))
 			g_print ("Could not set the printer to PDF\n");
 	}
-	gnome_print_config_set (config, "Settings.Output.Media.PhysicalSize", "USLetter");
-	gnome_print_job_get_page_size_from_config (config, &width, &height);
-
+	gnome_print_job_print_to_file (job, options_output ? options_output :
+				       (options_pdf ? "catalog.pdf" : "catalog.ps"));
+	gnome_print_config_get_page_size (config, &width, &height);
+	
 	map = gp_fontmap_get ();
 	list = g_slist_copy (map->fonts);
 	l = list;
 
-	font1 = gnome_font_find_closest ("Albany AMT Regular", FONT_NAME_SIZE);
-	g_print ("Printing with %s\n", gnome_font_get_name (font1));
-	font2 = gnome_font_find_closest ("Albany AMT Regular", FONT_INFO_SIZE);
-	
-	catalog_beginpage (gpc);
+	font1 = gnome_font_find_closest (FONT_NAME, FONT_NAME_SIZE);
+	font2 = gnome_font_find_closest (FONT_INFO, FONT_INFO_SIZE);
 
 	y = height - TOP_MARGIN;
 	x = 20;
 	row_size = 40;
+
+	catalog_beginpage (gpc);
+	y = generate_header (config, gpc, x, y);
 	
 	while (l) {
 		GPFontEntry *entry = l->data;
 		GnomeFont *font;
 		gchar *info;
 		
+		if ((options_max != -1) && (i == options_max))
+			break;
+		
 		if (y < BOTTOM_MARGIN) {
 			catalog_endpage (gpc);
 			catalog_beginpage (gpc);
 			y = height - TOP_MARGIN;
 		}
 
-
-		info = g_strdup_printf ("#%d [%s]", i++, entry->file);
+		info = g_strdup_printf ("#%d [%s]", i + 1, entry->file);
 		gnome_print_setfont (gpc, font1);
 		gnome_print_moveto (gpc, x, y);
 		gnome_print_show (gpc, entry->name);
@@ -318,10 +469,8 @@ generate_catalog (void)
 		
 		y -= row_size;
 		g_free (info);
-#if 0
-		if (i > 3)
-			break;
-#endif
+
+		i++;
 		l = l->next;
 	}
 
@@ -414,7 +563,7 @@ usage (gint num)
 		g_print ("Font number out of range\n");
 		break;
 	case 3:
-		g_print ("Option --generate needs a font number to be specified\n");
+		g_print ("Option --generate or --max need number to be specified\n");
 		break;
 	case 4:
 		g_print ("Option --output can only be used with the --generate or --catalog options\n");
@@ -445,7 +594,7 @@ check_options (int argc, const char *argv[])
 		num++;
 	if (options_dump_full)
 		num++;
-	if ((options_generate != -1) || retval == POPT_ERROR_NOARG)
+	if (options_generate != -1)
 		num++;
 	if (options_catalog)
 		num++;
@@ -496,7 +645,7 @@ main (int argc, const char *argv[])
 {
 	struct sigaction sig;
 
-	/* Catch SIGSEGV signals */
+	/* We need to catch crashes */
 	sig.sa_handler = handle_sigsegv;
 	sig.sa_flags = 0;
 	sigaction (SIGSEGV, &sig, NULL);
